@@ -29,6 +29,7 @@
 #include <string.h>
 #include <gconf/gconf-client.h>
 #include <libgnomeui/gnome-thumbnail.h>
+#include <libgnomeui/gnome-bg.h>
 
 typedef enum {
   GNOME_WP_SHADE_TYPE_SOLID,
@@ -112,6 +113,43 @@ get_selected_item (AppearanceData *data,
   return item;
 }
 
+static gboolean predicate (gpointer key, gpointer value, gpointer data)
+{
+  GnomeBG *bg = data;
+  GnomeWPItem *item = value;
+
+  return item->bg == bg;
+}
+
+static void on_item_changed (GnomeBG *bg, AppearanceData *data) {
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  GtkTreePath *path;
+  GnomeWPItem *item;
+
+  item = g_hash_table_find (data->wp_hash, predicate, bg);
+
+  if (!item)
+    return;
+  
+  model = gtk_tree_row_reference_get_model (item->rowref);
+  path = gtk_tree_row_reference_get_path (item->rowref);
+
+  if (gtk_tree_model_get_iter (model, &iter, path)) {
+    g_signal_handlers_block_by_func (bg, G_CALLBACK (on_item_changed), data);
+      
+    GdkPixbuf *pixbuf = gnome_wp_item_get_thumbnail (item, data->thumb_factory);
+    if (pixbuf) {
+      gtk_list_store_set (GTK_LIST_STORE (data->wp_model), &iter,
+                          0, pixbuf,
+                         -1);
+      g_object_unref (pixbuf);
+    }
+
+    g_signal_handlers_unblock_by_func (bg, G_CALLBACK (on_item_changed), data);
+  }
+}
+
 static void
 wp_props_load_wallpaper (gchar *key,
                          GnomeWPItem *item,
@@ -140,6 +178,7 @@ wp_props_load_wallpaper (gchar *key,
 
   path = gtk_tree_model_get_path (data->wp_model, &iter);
   item->rowref = gtk_tree_row_reference_new (data->wp_model, path);
+  g_signal_connect (item->bg, "changed", G_CALLBACK (on_item_changed), data);
   gtk_tree_path_free (path);
 }
 
@@ -401,24 +440,19 @@ wp_color_changed (AppearanceData *data,
   if (item == NULL)
     return;
 
-  g_free (item->pri_color);
-
   gtk_color_button_get_color (GTK_COLOR_BUTTON (data->wp_pcpicker), item->pcolor);
-
-  item->pri_color = gdk_color_to_string (item->pcolor);
-
-  g_free (item->sec_color);
-
   gtk_color_button_get_color (GTK_COLOR_BUTTON (data->wp_scpicker), item->scolor);
 
-  item->sec_color = gdk_color_to_string (item->scolor);
-
   if (update)
   {
-    gconf_client_set_string (data->client, WP_PCOLOR_KEY,
-                             item->pri_color, NULL);
-    gconf_client_set_string (data->client, WP_SCOLOR_KEY,
-                             item->sec_color, NULL);
+    gchar *pcolor, *scolor;
+
+    pcolor = gdk_color_to_string (item->pcolor);
+    scolor = gdk_color_to_string (item->scolor);
+    gconf_client_set_string (data->client, WP_PCOLOR_KEY, pcolor, NULL);
+    gconf_client_set_string (data->client, WP_SCOLOR_KEY, scolor, NULL);
+    g_free (pcolor);
+    g_free (scolor);
   }
 
   wp_shade_type_changed (NULL, data);
@@ -583,6 +617,7 @@ static gboolean
 wp_props_wp_set (AppearanceData *data, GnomeWPItem *item)
 {
   GConfChangeSet *cs;
+  gchar *pcolor, *scolor;
 
   cs = gconf_change_set_new ();
 
@@ -612,8 +647,12 @@ wp_props_wp_set (AppearanceData *data, GnomeWPItem *item)
 
   gconf_change_set_set_string (cs, WP_SHADING_KEY, item->shade_type);
 
-  gconf_change_set_set_string (cs, WP_PCOLOR_KEY, item->pri_color);
-  gconf_change_set_set_string (cs, WP_SCOLOR_KEY, item->sec_color);
+  pcolor = gdk_color_to_string (item->pcolor);
+  scolor = gdk_color_to_string (item->scolor);
+  gconf_change_set_set_string (cs, WP_PCOLOR_KEY, pcolor);
+  gconf_change_set_set_string (cs, WP_SCOLOR_KEY, scolor);
+  g_free (pcolor);
+  g_free (scolor);
 
   gconf_client_commit_change_set (data->client, cs, TRUE, NULL);
 
@@ -944,6 +983,7 @@ desktop_init (AppearanceData *data,
   GtkCellRenderer *cr;
   GtkFileFilter *filter;
   const gchar *pictures;
+  const gchar *start_dir;
 
   g_object_set (gtk_settings_get_default (), "gtk-tooltip-timeout", 500, NULL);
 
@@ -1074,15 +1114,26 @@ desktop_init (AppearanceData *data,
                      GTK_RESPONSE_OK,
                      NULL));
 
+  gtk_dialog_set_default_response (GTK_DIALOG (data->wp_filesel), GTK_RESPONSE_OK);
   gtk_file_chooser_set_select_multiple (data->wp_filesel, TRUE);
   gtk_file_chooser_set_use_preview_label (data->wp_filesel, FALSE);
 
+  start_dir = g_get_home_dir ();
+
+  if (g_file_test ("/usr/share/backgrounds", G_FILE_TEST_IS_DIR)) {
+    gtk_file_chooser_add_shortcut_folder (data->wp_filesel,
+                                          "/usr/share/backgrounds", NULL);
+    start_dir = "/usr/share/backgrounds";
+  }
+
   pictures = g_get_user_special_dir (G_USER_DIRECTORY_PICTURES);
   if (pictures != NULL && g_file_test (pictures, G_FILE_TEST_IS_DIR)) {
     gtk_file_chooser_add_shortcut_folder (data->wp_filesel, pictures, NULL);
-    gtk_file_chooser_set_current_folder (data->wp_filesel, pictures);
+    start_dir = pictures;
   }
 
+  gtk_file_chooser_set_current_folder (data->wp_filesel, start_dir);
+
   filter = gtk_file_filter_new ();
   gtk_file_filter_add_pixbuf_formats (filter);
   gtk_file_filter_set_name (filter, _("Images"));
