@@ -36,8 +36,8 @@
 #include "gconf-property-editor.h"
 #include "activate-settings-daemon.h"
 #include "capplet-stock-icons.h"
-#include <../accessibility/keyboard/accessibility-keyboard.h>
 
+#include "gnome-keyboard-properties-a11y.h"
 #include "gnome-keyboard-properties-xkb.h"
 
 enum {
@@ -76,12 +76,6 @@ create_dialog (void)
 				   WID ("cursor_blink_time_scale"));
 	g_object_unref (G_OBJECT (size_group));
 
-	image =
-	    gtk_image_new_from_stock (GTK_STOCK_JUMP_TO,
-				      GTK_ICON_SIZE_BUTTON);
-	gtk_button_set_image (GTK_BUTTON (WID ("accessibility_button")),
-			      image);
-
 	image =
 	    gtk_image_new_from_stock (GTK_STOCK_ADD, GTK_ICON_SIZE_BUTTON);
 	gtk_button_set_image (GTK_BUTTON (WID ("xkb_layouts_add")), image);
@@ -121,19 +115,6 @@ blink_to_widget (GConfPropertyEditor * peditor, const GConfValue * value)
 	return new_value;
 }
 
-static void
-accessibility_button_clicked (GtkWidget * widget, gpointer data)
-{
-	GError *err = NULL;
-	if (!g_spawn_command_line_async
-	    ("gnome-accessibility-keyboard-properties", &err))
-		capplet_error_dialog (GTK_WINDOW
-				      (gtk_widget_get_toplevel (widget)),
-				      _
-				      ("There was an error launching the keyboard tool: %s"),
-				      err);
-}
-
 static void
 dialog_response (GtkWidget * widget,
 		 gint response_id, GConfChangeSet * changeset)
@@ -141,8 +122,6 @@ dialog_response (GtkWidget * widget,
 	if (response_id == GTK_RESPONSE_HELP)
 		capplet_help (GTK_WINDOW (widget),
 			      "user-guide.xml", "goscustperiph-2");
-	else if (response_id == 0)
-		accessibility_button_clicked (NULL, NULL);
 	else
 		gtk_main_quit ();
 }
@@ -207,6 +186,7 @@ setup_dialog (GladeXML * dialog, GConfChangeSet * changeset)
 				   child), TRUE);
 
 	setup_xkb_tabs (dialog, changeset);
+	setup_a11y_tabs (dialog, changeset);
 }
 
 static void
@@ -245,18 +225,6 @@ get_legacy_settings (void)
 	g_object_unref (G_OBJECT (client));
 }
 
-#if 0
-static void
-setup_accessibility (GladeXML * dialog, GConfChangeSet * changeset)
-{
-	GtkWidget *notebook = WID ("notebook1");
-	GtkWidget *label =
-	    gtk_label_new_with_mnemonic (_("_Accessibility"));
-	GtkWidget *page = setup_accessX_dialog (changeset, FALSE);
-	gtk_notebook_append_page (GTK_NOTEBOOK (notebook), page, label);
-}
-#endif
-
 int
 main (int argc, char **argv)
 {
@@ -268,6 +236,7 @@ main (int argc, char **argv)
 	static gboolean apply_only = FALSE;
 	static gboolean get_legacy = FALSE;
 	static gboolean switch_to_typing_break_page = FALSE;
+	static gboolean switch_to_a11y_page = FALSE;
 
 	static GOptionEntry cap_options[] = {
 		{"apply", 0, 0, G_OPTION_ARG_NONE, &apply_only,
@@ -286,6 +255,11 @@ main (int argc, char **argv)
 		 N_
 		 ("Start the page with the typing break settings showing"),
 		 NULL},
+		{"a11y", 0, 0, G_OPTION_ARG_NONE,
+		 &switch_to_a11y_page,
+		 N_
+		 ("Start the page with the accessibility settings showing"),
+		 NULL},
 		{NULL}
 	};
 
@@ -323,8 +297,16 @@ main (int argc, char **argv)
 			gtk_notebook_set_current_page (GTK_NOTEBOOK
 						       (WID
 							("keyboard_notebook")),
-						       3);
+						       4);
 		}
+		else if (switch_to_a11y_page) {
+			gtk_notebook_set_current_page (GTK_NOTEBOOK
+						       (WID
+							("keyboard_notebook")),
+						       2);
+
+		}
+
 		capplet_set_icon (WID ("keyboard_dialog"),
 				  "gnome-dev-keyboard");
 		gtk_widget_show (WID ("keyboard_dialog"));
